#include <jni.h>
#include <Carbon/Carbon.h>
#include <Cocoa/Cocoa.h>

@interface EventsApplication : NSApplication 
@end

@implementation EventsApplication

BOOL mouse1Clicked;
BOOL mouse2Clicked;
BOOL mouse3Clicked;

+ (CGPoint)getMousePointWithDeltaX:(int)x deltaY:(int)y
{
	CGPoint point;
	//Point globalPoint;
	CGDirectDisplayID displayID;
	CGDisplayCount displayCnt;
	CGRect disp = CGDisplayBounds(kCGDirectMainDisplay);
	
    // mbl: 
	//  GetGlobalMouse(&globalPoint);
    // is obsolete. Here's a trick to get the cursor position:
    // http://unlogica.com/blog/2008/02/and_now_how_to_determine_the_m.html
    CGEventRef ourEvent = CGEventCreate(NULL);
    CGPoint globalPoint = CGEventGetLocation(ourEvent);
//NSLog(@"Location? %f, %f", (float)globalPoint.x, (float)globalPoint.y);
    CFRelease(ourEvent);
    
	point = globalPoint;
	if (x == 0 && y == 0)
		return point;
	
	point.x += x;
	point.y += y;
	if (CGGetDisplaysWithPoint(point, 1, &displayID, &displayCnt) != 0 || displayCnt < 1) {
		CGPoint oldPoint;
		oldPoint = globalPoint;
		if (CGGetDisplaysWithPoint(oldPoint, 1, &displayID, &displayCnt) == 0 && displayCnt > 0)
			disp = CGDisplayBounds(displayID);
		if (point.x < disp.origin.x) point.x = disp.origin.x;
		else if (disp.origin.x + disp.size.width - 1 < point.x) point.x = disp.origin.x + disp.size.width - 1;
		if (point.y < disp.origin.y) point.y = disp.origin.y;
		else if (disp.origin.y + disp.size.height - 1 < point.y) point.y = disp.origin.y + disp.size.height - 1;
	}
	
	return point;
}

+ (CGKeyCode)getKeyCode:(CGCharCode)charCode {
	CGKeyCode keyCode;
	keyCode=0x00;
	
	OSStatus err;
	KeyboardLayoutRef ourKeyboardLayout = NULL;
	
//NSLog(@"Searching keycode for UTF char: %d", (int)charCode);
	
	// Get the real keycode based on the curent keymap
	KeyboardLayoutKind layoutKind;
	UCKeyboardLayout *uchrHandle = NULL;
	const void *kchrHandle = NULL;
	//KeyScript();
	err = KLGetCurrentKeyboardLayout ( & ourKeyboardLayout ); // only since 10.2, in Keyboards.h
//NSLog(@"Current layout is: %0X", ourKeyboardLayout);
	KLGetKeyboardLayoutProperty(ourKeyboardLayout, kKLKind, (const void **) &layoutKind);
	if (layoutKind == kKLKCHRuchrKind || layoutKind == kKLuchrKind)
		KLGetKeyboardLayoutProperty(ourKeyboardLayout, kKLuchrData, (const void **) &uchrHandle);
	else
		KLGetKeyboardLayoutProperty(ourKeyboardLayout, kKLKCHRData, (const void **) &kchrHandle);
	if (uchrHandle) {
		UInt16 ikeyCode;
		UInt32 keyboardType = LMGetKbdType();
		UInt32 deadKeyState = 0;
		UniCharCount actualStringLength;
		UniChar unicodeChar;
		// Iterate Over Each Key Code
		for (ikeyCode = 0; ikeyCode < 127; ikeyCode++) {
			err = UCKeyTranslate (uchrHandle, ikeyCode, kUCKeyActionDown, 0, keyboardType, kUCKeyTranslateNoDeadKeysBit, &deadKeyState, 1, &actualStringLength, &unicodeChar);
			if (err == noErr && (int)charCode == (int)unicodeChar) {
				keyCode = ikeyCode;
//NSLog(@"Got keycode: %d", (int)keyCode);
				ikeyCode = 128;
			}
		}
	}
	else if (kchrHandle) {
		UInt16 ikeyCode;
		UInt32 state=0;
		UInt32 kchrCharacters;
		// Iterate Over Each Key Code
		for (ikeyCode = 0; ikeyCode < 127; ikeyCode++) {
			kchrCharacters = KeyTranslate(kchrHandle, ikeyCode, &state);
			if (kchrCharacters == charCode) {
				keyCode = ikeyCode;
//NSLog(@"Got keycode: %0X", (int)keyCode);
				ikeyCode = 128;
			}
		}
	}
	else {
		keyCode=0x23;
//NSLog(@"Unable To Determine Key Map - Reverting to US Mapping\n");
	}
	
	return keyCode;
}

/*
** Start up: use poseAsClass to subclass the NSApplication object on the fly.
*/
JNIEXPORT jboolean JNICALL Java_fr_emn_osx_OSXEvents_startup(JNIEnv *env, jclass cl) {
    [EventsApplication poseAsClass: [NSApplication class]];
	return TRUE;
}

/*
** Shut down: release our data.
*/
JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_shutdown(JNIEnv *env, jobject this) {

}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_mouseMove(JNIEnv *env, jclass cl, jint dx, jint dy) {
	CGPoint point = [EventsApplication getMousePointWithDeltaX:dx deltaY:dy];
	CGEventType type;
	CGMouseButton button;
	
	if (mouse1Clicked) {
		type = kCGEventLeftMouseDragged;
		button = kCGMouseButtonLeft;
	} else if (mouse2Clicked) {
		type = kCGEventRightMouseDragged;
		button = kCGMouseButtonRight;
	} else if (mouse3Clicked) {
		type = kCGEventOtherMouseDragged;
		button = kCGMouseButtonCenter;
	} else {
		type = kCGEventMouseMoved;
		button = kCGMouseButtonLeft;
	}
	
	CFRelease(CGEventCreate(NULL));
	CGEventRef event = CGEventCreateMouseEvent(NULL, type, point, button);
	CGEventSetType(event, type);
	CGEventPost(kCGSessionEventTap, event);
	CFRelease(event);
}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_mouseMoveTo(JNIEnv *env, jclass cl, jint x, jint y) {
	CGPoint point = CGPointMake(x, y);
	CGEventType type;
	CGMouseButton button;
	
	if (mouse1Clicked) {
		type = kCGEventLeftMouseDragged;
		button = kCGMouseButtonLeft;
	} else if (mouse2Clicked) {
		type = kCGEventRightMouseDragged;
		button = kCGMouseButtonRight;
	} else if (mouse3Clicked) {
		type = kCGEventOtherMouseDragged;
		button = kCGMouseButtonCenter;
	} else {
		type = kCGEventMouseMoved;
		button = kCGMouseButtonLeft;
	}
	
	CFRelease(CGEventCreate(NULL));
	CGEventRef event = CGEventCreateMouseEvent(NULL, type, point, button);
	CGEventSetType(event, type);
	CGEventPost(kCGSessionEventTap, event);
	CFRelease(event);
}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_mouseDown (JNIEnv *env, jclass cl, jint mouseNum, jint clickCount) {
	CGPoint point = [EventsApplication getMousePointWithDeltaX:0 deltaY:0];
	CGEventType type;
	CGMouseButton button;
	
	if (mouseNum == 0) {
		type = kCGEventLeftMouseDown;
		button = kCGMouseButtonLeft;
		mouse1Clicked = YES;
	} else if (mouseNum == 1) {
		type = kCGEventRightMouseDown;
		button = kCGMouseButtonRight;
		mouse2Clicked = YES;
	} else {
		type = kCGEventOtherMouseDown;
		button = kCGMouseButtonCenter;
		mouse3Clicked = YES;
	}
	CFRelease(CGEventCreate(NULL));
	CGEventRef event = CGEventCreateMouseEvent(NULL, type, point, button);
	CGEventSetIntegerValueField(event, kCGMouseEventClickState, clickCount);
	CGEventSetType(event, type);
	CGEventPost(kCGSessionEventTap, event);
	CFRelease(event);
}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_mouseUp (JNIEnv *env, jclass cl, jint mouseNum, jint clickCount) {
	CGPoint point = [EventsApplication getMousePointWithDeltaX:0 deltaY:0];
	CGEventType type;
	CGEventType typedown;
	CGMouseButton button;
	
    if (mouseNum == 0) {
		type = kCGEventLeftMouseUp;
		typedown = kCGEventLeftMouseDown;
		button = kCGMouseButtonLeft;
		mouse1Clicked = NO;
	} else if (mouseNum == 1) {
		type = kCGEventRightMouseUp;
		typedown = kCGEventRightMouseDown;
		button = kCGMouseButtonRight;
		mouse2Clicked = NO;
	} else {
		type = kCGEventOtherMouseUp;
		typedown = kCGEventOtherMouseDown;
		button = kCGMouseButtonCenter;
		mouse3Clicked = NO;
	}
	CFRelease(CGEventCreate(NULL));
	CGEventRef event = CGEventCreateMouseEvent(NULL, type, point, button);
	CGEventSetIntegerValueField(event, kCGMouseEventClickState, clickCount);
	CGEventSetType(event, type);
	CGEventPost(kCGSessionEventTap, event);
#if (MAC_OS_X_VERSION_MAX_ALLOWED < MAC_OS_X_VERSION_10_5)
	if (clickCount > 1 && mouseNum == 0) {
		for (int i = 1; i < clickCount; i++) {
			CGEventSetType(event, typedown);
			CGEventSetIntegerValueField(event, kCGMouseEventClickState, clickCount);
			CGEventPost(kCGSessionEventTap, event);
			CGEventSetType(event, type);
			CGEventSetIntegerValueField(event, kCGMouseEventClickState, clickCount);
			CGEventPost(kCGSessionEventTap, event);
		}
	}
#endif
	CFRelease(event);
}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_scrollWheel (JNIEnv *env, jclass cl, jint w, jint z) {
	int32_t countW = -w;
	int32_t countZ = -z;
	
#if (MAC_OS_X_VERSION_MAX_ALLOWED >= MAC_OS_X_VERSION_10_5)
	if (countW != 0 || countZ != 0) {
		CFRelease(CGEventCreate(NULL));
		CGEventRef event = CGEventCreateScrollWheelEvent(NULL, kCGScrollEventUnitPixel, 2, countZ, countW);
		CGEventSetType(event, kCGEventScrollWheel);
		CGEventPost(kCGSessionEventTap, event);
		CFRelease(event);
	}
#else
#define kAdhocFactor 16
	accumuW += countW;
	countW = accumuW / kAdhocFactor;
	accumuW = accumuW % kAdhocFactor;
	accumuZ += countZ;
	countZ = accumuZ / kAdhocFactor;
	accumuZ = accumuZ %kAdhocFactor;
	if (countW != 0 || countZ != 0) {
		CFRelease(CGEventCreate(NULL));
		CGEventRef event = CGEventCreate(NULL);
		CGEventSetType(event, kCGEventScrollWheel);
		CGEventSetIntegerValueField(event, kCGScrollWheelEventDeltaAxis1, countZ);
		CGEventSetIntegerValueField(event, kCGScrollWheelEventDeltaAxis2, countW);
		CGEventSetIntegerValueField(event, kCGScrollWheelEventInstantMouser, 0);
		CGEventSetType(event, kCGEventScrollWheel);
		CGEventPost(kCGSessionEventTap, event);
		CFRelease(event);
	}
#endif
}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_keyPress (JNIEnv *env, jclass cl, jint keyCode) {
	CFRelease(CGEventCreate(NULL));
	CGEventRef event = CGEventCreateKeyboardEvent(NULL, (CGKeyCode)keyCode, TRUE);
	CGEventSetType(event, kCGEventKeyDown);
	CGEventPost(kCGSessionEventTap, event);
	CFRelease(event);
}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_keyRelease (JNIEnv *env, jclass cl, jint keyCode) {
	CFRelease(CGEventCreate(NULL));
	CGEventRef event = CGEventCreateKeyboardEvent(NULL, (CGKeyCode)keyCode, FALSE);
	CGEventSetType(event, kCGEventKeyUp);
	CGEventPost(kCGSessionEventTap, event);
	CFRelease(event);
}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_keyChar (JNIEnv *env, jclass cl, jint charCode) {
	CGKeyCode keyCode = [EventsApplication getKeyCode:(CGCharCode)charCode];
	
	CFRelease(CGEventCreate(NULL));
	CGEventRef event = CGEventCreateKeyboardEvent(NULL, (CGKeyCode)keyCode, TRUE);
	CGEventSetType(event, kCGEventKeyDown);
	CGEventPost(kCGSessionEventTap, event);
	CFRelease(event);
	CFRelease(CGEventCreate(NULL));
	CGEventRef event2 = CGEventCreateKeyboardEvent(NULL, (CGKeyCode)keyCode, FALSE);
	CGEventSetType(event2, kCGEventKeyUp);
	CGEventPost(kCGSessionEventTap, event2);
	CFRelease(event2);

}

JNIEXPORT void JNICALL Java_fr_emn_osx_OSXEvents_keyString (JNIEnv *env, jclass cl, jstring string) {
	const char *stkeys  = (*env)->GetStringUTFChars(env, string, 0);

	int i = 0;
	for (i = 0; stkeys[i]; i++) {
	
		CGKeyCode keyCode = [EventsApplication getKeyCode:stkeys[i]];
		
		CFRelease(CGEventCreate(NULL));
		CGEventRef event = CGEventCreateKeyboardEvent(NULL, (CGKeyCode)keyCode, TRUE);
		CGEventSetType(event, kCGEventKeyDown);
		CGEventPost(kCGSessionEventTap, event);
		CFRelease(event);
		CFRelease(CGEventCreate(NULL));
		CGEventRef event2 = CGEventCreateKeyboardEvent(NULL, (CGKeyCode)keyCode, FALSE);
		CGEventSetType(event2, kCGEventKeyUp);
		CGEventPost(kCGSessionEventTap, event2);
		CFRelease(event2);
		
	}
	
	(*env)->ReleaseStringUTFChars(env, string, stkeys);

}

@end
