package fr.lri.insitu.FlowStates.examples.graphicalEditor;

import java.awt.Color;
import java.awt.Event;
import java.awt.geom.Point2D;
import java.io.File;

import fr.lri.insitu.FlowStates.examples.graphicalEditor.events.RubberBand;
import fr.lri.insitu.FlowStates.examples.graphicalEditor.events.SelectToolPalette;
import fr.lri.insitu.FlowStates.examples.menus.MenuItem;
import fr.lri.insitu.FlowStates.examples.menus.events.PalettePosition;
import fr.lri.insitu.FlowStates.sm.IConStateMachine;
import fr.lri.insitu.FlowStates.transitions.SwitchOnPosition;
import fr.lri.insitu.FlowStates.transitions.SwitchOnTag;
import fr.lri.swingstates.canvas.CEllipse;
import fr.lri.swingstates.canvas.CExtensionalTag;
import fr.lri.swingstates.canvas.CPolyLine;
import fr.lri.swingstates.canvas.CRectangle;
import fr.lri.swingstates.canvas.CShape;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.EventOnPosition;

abstract class Tool extends CExtensionalTag {
	protected CShape shapeBuilt;
	protected Point2D ptInit;
	public Tool() {
		super();
	}

	public abstract CShape getShape(Point2D pt);
	public abstract void addPoint(Point2D pt);
}

class Form extends CExtensionalTag {
	public Form() {
		super();
	}
}

public class Palette {

	protected ZoomableCanvas canvas;
	protected int toolSide = 40;
	protected CExtensionalTag tools = new CExtensionalTag() { };
	protected CShape backgroundPalette;
	protected Form tagForms;
	protected Tool rectangle, ellipse, freeHand;
	
	protected IConStateMachine interactionPalette, interactionToolglass;
	
	public Palette(ZoomableCanvas canvas) {
		this.canvas = canvas;
		tagForms = new Form();
		rectangle = new Tool() {
			public CShape getShape(Point2D pt) {
				ptInit = ((ZoomableCanvas)getCanvas()).getCoordinates(pt);
				shapeBuilt = (CShape) Palette.this.canvas.newRectangle(ptInit);
				shapeBuilt.addTag(tagForms);
				return shapeBuilt;
			}
			public void addPoint(Point2D pt) {
				Point2D transformed = ((ZoomableCanvas)getCanvas()).getCoordinates(pt);
				((CRectangle)shapeBuilt).setDiagonal(ptInit, transformed);
			}
		};
		ellipse = new Tool() {
			public CShape getShape(Point2D pt) {
				ptInit = ((ZoomableCanvas)getCanvas()).getCoordinates(pt);
				shapeBuilt = (CShape) Palette.this.canvas.newEllipse(ptInit);
				shapeBuilt.addTag(tagForms);
				return shapeBuilt;
			}
			public void addPoint(Point2D pt) {
				Point2D transformed = ((ZoomableCanvas)getCanvas()).getCoordinates(pt);
				((CEllipse)shapeBuilt).setDiagonal(ptInit, transformed);
			}
		};
		freeHand = new Tool() {
			public CShape getShape(Point2D pt) {
				ptInit = ((ZoomableCanvas)getCanvas()).getCoordinates(pt);
				shapeBuilt = (CShape) Palette.this.canvas.newFreeHand(ptInit).setFilled(false);
				shapeBuilt.addTag(tagForms);
				return shapeBuilt;
			}
			public void addPoint(Point2D pt) {
				Point2D transformed = ((ZoomableCanvas)getCanvas()).getCoordinates(pt);
				((CPolyLine)shapeBuilt).lineTo(transformed);
			}
		};

		int insets = 0;
		backgroundPalette = (CShape) canvas.newRectangle(0, 0, 3 * toolSide, toolSide).setFillPaint(Color.WHITE).setPickable(false).addTag(tools);
		canvas.newImage(insets, insets, "pics"+File.separator+"rectangle.png").addTag(rectangle).addTag(tools);
		canvas.newImage(toolSide+insets, insets, "pics"+File.separator+"ellipse.png").addTag(ellipse).addTag(tools);
		canvas.newImage(2*toolSide+insets, insets, "pics"+File.separator+"freehand.png").addTag(freeHand).addTag(tools);
		tools.setParent(backgroundPalette);
		
		
		interactionPalette = new IConStateMachine("palette", canvas) {
			Tool currentTool;
			CShape dragged = null;
			
			State noTool = new State() {
				Transition selectOtherTool = new SwitchOnTag(Tool.class, SelectToolPalette.class, SWITCH_ON, ">> toolOn") {
					public void action() { currentTool = (Tool)getTag(); }
				};
				
				Transition selectShape = new SwitchOnTag(Form.class, RubberBand.class, SWITCH_ON, ">> drag") { 
					public void action() {
						dragged = getShape();
					}
				};
			};

			State toolOn = new State() {
				Transition selectOtherTool = new SwitchOnTag(Tool.class, SelectToolPalette.class, SWITCH_ON) {
					public void action() { currentTool = (Tool)getTag(); }
				};
					
				Transition selectShape = new SwitchOnTag(Form.class, RubberBand.class, SWITCH_ON, ">> drag") {
					public void action() {
						dragged = getShape();
					}
				};
				
				Transition startDrawing = new SwitchOnPosition(RubberBand.class, SWITCH_ON, ">> draw") {
					public void action() { currentTool.getShape(getPoint()); }
				};
			};
			
			State drag = new State() {
				
				Transition endDrag = new SwitchOnPosition(RubberBand.class, SWITCH_OFF, ">> noTool") {
					public void action() {
						dragged = null;
					}
				};
				
				Transition drag = new EventOnPosition(RubberBand.class) {
					public void action() {
						dragged.translateTo(getPoint().getX(), getPoint().getY());
					}
				};
			};
			
			State draw = new State() {
				Transition endDrawing = new SwitchOnPosition(RubberBand.class, SWITCH_OFF, ">> toolOn") { };
				Transition addPoint = new EventOnPosition(RubberBand.class) {
					public void action() { currentTool.addPoint(getPoint()); }
				};
			};
		};
		
		interactionToolglass = new IConStateMachine("toolglass", canvas) {
			Tool currentTool;
			
			// To change a palette into a toolglass, we have to:
			// - remove the first state
			// - fusion the transitions to change current tool and startDrawing into a single one
			// - implement a moving behavior to the palette --> a new kind of state
			
			class StateTG extends State {
				Transition move = new EventOnPosition(PalettePosition.class) {
					public void action() {
						Palette.this.setTransparencyLevel(0.3f);
						Palette.this.moveTo(getPoint().getX(), getPoint().getY(), 0, 0);
					}
				};
			}
			
			State toolOn = new StateTG() {
				Transition startDrawing = new SwitchOnTag(Tool.class, RubberBand.class, SWITCH_ON, ">> draw") {
					public void action() { currentTool = (Tool)getTag(); currentTool.getShape(getPoint()); }
				};
			};
			State draw = new StateTG() {
				Transition endDrawing = new SwitchOnPosition(RubberBand.class, SWITCH_OFF, ">> toolOn") { };
				Transition addPoint = new EventOnPosition(RubberBand.class) {
					public void action() { currentTool.addPoint(getPoint()); }
				};
			};
		};
	}

	public IConStateMachine getInteractionToolglass() {
		return interactionToolglass;
	}
	
	
	protected void moveTo(double x, double y, double rx, double ry) {
		CRectangle bb = backgroundPalette.getBoundingBox();
		double currentRx = bb.getMinX() + rx * bb.getWidth();
		double currentRy = bb.getMinY() + ry * bb.getWidth();
		backgroundPalette.translateBy(x-currentRx, y-currentRy);
	}

	public void setTransparencyLevel(float f) {
		tools.setTransparencyFill(f);
		tools.setTransparencyOutline(f);
	}

	public IConStateMachine getInteractionPalette() {
		return interactionPalette;
	}
}

