package fr.lri.insitu.FlowStates.examples.menus;

import java.awt.Color;
import java.awt.Font;
import java.awt.Point;
import java.awt.geom.Point2D;

import fr.lri.swingstates.canvas.CExtensionalTag;
import fr.lri.swingstates.canvas.CShape;
import fr.lri.swingstates.canvas.Canvas;

public class MenuLayout {
	public static final Color  TRANSPARENT_COLOR      = new Color(0,0,0,0);
	public static final Color  BG_COLOR_DEFAULT       = new Color(119, 150, 214);
	public static final Color  ITEMS_BG_COLOR_DEFAULT       = TRANSPARENT_COLOR;
	public static final Color  HILITE_COLOR_DEFAULT   = new Color(129,18,199,100);
	public static final Color  BORDER_COLOR_DEFAULT      = new Color(244, 247, 254, 140);
	public static final Color  SHADOW_COLOR_DEFAULT      = new Color(100,100,100,60);
	public static final Color  FONT_COLOR_DEFAULT      = Color.WHITE;
	public static final Font   FONT              = new Font("verdana", Font.BOLD, 16);
	
	protected CShape parent, bgShape;
	protected CExtensionalTag tagWhole, tagLabels, tagBg, tagItemBg;
	
	protected Canvas canvas;

	protected Point2D currentPos = new Point2D.Double();
	
	protected CExtensionalTag parentShape;
	
	protected Color backgroundColor = BG_COLOR_DEFAULT;
	protected Color itemsBackgroundColor = ITEMS_BG_COLOR_DEFAULT;
	protected Color hiliteColor = HILITE_COLOR_DEFAULT;
	protected Color borderColor = BORDER_COLOR_DEFAULT;
	protected Color shadowColor = SHADOW_COLOR_DEFAULT;
	protected Color fontColor = FONT_COLOR_DEFAULT;
	
	public MenuLayout(Canvas canvas) {
		this.canvas = canvas;
		tagWhole = new CExtensionalTag(canvas) { };
		tagLabels = new CExtensionalTag(canvas) { };
		tagBg = new CExtensionalTag(canvas) { };
		tagItemBg = new CExtensionalTag(canvas) { };
		parentShape = new CExtensionalTag() { };
	}

	public void activeMenu(Point2D pt) {
		currentPos.setLocation(pt);
		parent.translateTo(pt.getX(), pt.getY()).setDrawable(true);
		tagWhole.setPickable(true);
		tagWhole.aboveAll();
		tagLabels.aboveAll();
		tagLabels.setPickable(false);
	}
	
	public void showMenu() {
		tagWhole.setDrawable(true).setPickable(true);
		tagWhole.aboveAll();
		tagLabels.aboveAll();
		tagLabels.setPickable(false);
	}
	
	public void showMenu(Point2D pt) {
		currentPos.setLocation(pt);
		parent.translateTo(pt.getX(), pt.getY()).setDrawable(true);
		showMenu();
	}

	public void moveBy(double dx, double dy) {
		currentPos.setLocation(currentPos.getX()+dx, currentPos.getY()+dy);
		parent.translateBy(dx, dy);
	}
	
	public void hideMenu() {
		removeHighlight();
		tagWhole.setDrawable(false).setPickable(false);
	}

	public void removeHighlight() {
		tagItemBg.setFillPaint(itemsBackgroundColor);
	}

	public CExtensionalTag getParentShape() {
		return parentShape;
	}
	
	public Point2D getCurrentPosition() {
		return currentPos;
	}

	public Color getBackgroundColor() {
		return backgroundColor;
	}

	public void setBackgroundColor(Color backgroundColor) {
		this.backgroundColor = backgroundColor;
		tagBg.setFillPaint(backgroundColor);
	}

	public Color getHiliteColor() {
		return hiliteColor;
	}

	public void setHiliteColor(Color hiliteColor) {
		this.hiliteColor = hiliteColor;
	}

	public Color getBorderColor() {
		return borderColor;
	}

	public void setBorderColor(Color borderColor) {
		this.borderColor = borderColor;
		tagBg.setOutlinePaint(borderColor);
	}

	public Color getShadowColor() {
		return shadowColor;
	}

	public void setShadowColor(Color shadowColor) {
		this.shadowColor = shadowColor;
	}

	public Color getFontColor() {
		return fontColor;
	}

	public void setFontColor(Color fontColor) {
		this.fontColor = fontColor;
		tagLabels.setFillPaint(fontColor);
	}

	public Color getItemsBackgroundColor() {
		return itemsBackgroundColor;
	}

	public void setItemsBackgroundColor(Color itemsBackgroundColor) {
		this.itemsBackgroundColor = itemsBackgroundColor;
	}
}
