package fr.lri.insitu.FlowStates.examples.menus;

import java.awt.geom.Point2D;

import javax.swing.JFrame;

import fr.lri.insitu.FlowStates.events.OutSlotEvent;
import fr.lri.insitu.FlowStates.examples.graphicalEditor2.Pan;
import fr.lri.insitu.FlowStates.examples.graphicalEditor2.Zoom;
import fr.lri.insitu.FlowStates.examples.menus.events.Control;
import fr.lri.insitu.FlowStates.examples.menus.events.InRange;
import fr.lri.insitu.FlowStates.sm.IConStateMachine;
import fr.lri.insitu.FlowStates.transitions.Switch;
import fr.lri.insitu.FlowStates.transitions.SwitchOnPosition;
import fr.lri.insitu.FlowStates.transitions.SwitchOnTag;
import fr.lri.swingstates.canvas.Canvas;
import fr.lri.swingstates.canvas.transitions.EventOnTag;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.EventOnPosition;

public class TrackingMenu extends IConStateMachine {

	protected CircularMenuLayout menu;
	protected MenuItem currentItem;
	protected Point2D lastPosition, startPosition;

	protected String[] properties = {"Radius"};

	public TrackingMenu(Canvas canvas, JFrame frame, String[] items) {
		this(canvas, frame, 100, 30, items);
	}

	public TrackingMenu(Canvas canvas, JFrame frame, int radius, int innerRadius, String[] items) {
		super("trackingMenu", canvas);
		menu = new CircularMenuLayout(canvas, radius, innerRadius, true, items);
		menu.showMenu(new Point2D.Double(0, 0));
	}

	@Override
	public String[] getProperties() {
		return properties;
	}

	public int getRadius() {
		return menu.getRadius();
	}

	public void setRadius(int r) {
		menu.setRadius(r);
	}

	public Class<? extends OutSlotEvent>[] getOutputTypes() {
		return new Class[]{Pan.class,Zoom.class};
	}

	State outOfRange = new State() {
		Transition startTrackingInMenu = new SwitchOnTag(MenuItem.class, InRange.class, SWITCH_ON, ">> tracking") { };
		Transition startTrackingOutMenu = new SwitchOnPosition(InRange.class, SWITCH_ON, ">> tracking") {
			public void action() {
				menu.showMenu(getPoint());
			}
		};
	};
	State tracking = new State() {
		Transition stopTracking = new Switch(InRange.class, SWITCH_OFF, ">> outOfRange") {
			public void action() {
				menu.removeHighlight();
				menu.hideMenu();
			}
		};

		Transition startControl = new SwitchOnTag(MenuItem.class, Control.class, SWITCH_ON, ">> touching") {
			public void action() {
				currentItem = (MenuItem)getTag();
				lastPosition = getPoint();
				startPosition = getPoint();
			}
		};

		// tracking menu does not move, highlight menu items
		Transition moveInMenu = new EventOnTag(MenuItem.class, InRange.class) {
			public void action() {
				if(currentItem == getTag()) return;
				menu.removeHighlight();
				currentItem = (MenuItem)getTag();
				currentItem.setFillPaint(menu.getHiliteColor());
			}
		};
		Transition moveOutMenu = new EventOnPosition(InRange.class) {
			double distanceFromMenuCenter;
			public boolean guard () {
				distanceFromMenuCenter = menu.getCurrentPosition().distance(getPoint());
				return distanceFromMenuCenter > menu.getRadius();
			}
			public void action() {
				double sinAngleToCenter = (getPoint().getY() - menu.getCurrentPosition().getY()) / distanceFromMenuCenter;
				double angleToCenter = Math.asin(sinAngleToCenter);
				double distanceToOutline = distanceFromMenuCenter - menu.getRadius();
				double dx = distanceToOutline * Math.cos(angleToCenter);
				if((getPoint().getX() - menu.getCurrentPosition().getX()) < 0) dx = -dx;
				double dy = distanceToOutline * sinAngleToCenter;

				menu.moveBy(dx, dy);
			}
		};
	};
	State touching = new State() {
		Transition stopControl = new SwitchOnPosition(Control.class, SWITCH_OFF, ">> tracking") {
			public void action() {
				menu.showMenu(getPoint());
			}
		};

		// tracking menu does not move
		Transition control = new EventOnPosition(Control.class) {
			public void action() {
				if (currentItem != null) {
					if(currentItem.getName().compareTo("zoom") == 0) {
						Zoom event = new Zoom();
						event.setSlotX(startPosition.getX());
						event.setSlotY(startPosition.getY());
						double deltaY = getPoint().getY() - lastPosition.getY();
						event.setSlotdZ(deltaY);
						fireEvent(event);
					} else {
						if(currentItem.getName().compareTo("pan") == 0) {
							Pan event = new Pan();
							event.setSlotDeltaX((int)(getPoint().getX() - lastPosition.getX()));
							event.setSlotDeltaY((int)(getPoint().getY() - lastPosition.getY()));
							fireEvent(event);
						}
					}
					lastPosition = getPoint();
				}
			}
		};
	};

}

