package fr.lri.insitu.FlowStates.examples.toolglass;

import java.awt.Color;

import fr.lri.swingstates.canvas.CExtensionalTag;
import fr.lri.swingstates.canvas.CRectangle;
import fr.lri.swingstates.canvas.CRectangularShape;
import fr.lri.swingstates.canvas.Canvas;

public class Palette {

	protected Canvas canvas;
	protected int toolSide = 40;
	protected CExtensionalTag background = new CExtensionalTag() { };
	protected CExtensionalTag foreground = new CExtensionalTag() { };
	
	public Palette(Canvas canvas) {
		Tool rectangle = new Tool() {
			public CRectangularShape getShape(double x, double y) {
				return (CRectangularShape) canvas.newRectangle(x, y, 1, 1).below(background);
			}
		};
		Tool ellipse = new Tool() {
			public CRectangularShape getShape(double x, double y) {
				return (CRectangularShape) canvas.newEllipse(x, y, 1, 1).below(background);
			}
		};
		Tool segment = new Tool() {
			public CRectangularShape getShape(double x, double y) {
				return (CRectangularShape) canvas.newSegment(x, y, x+1, y+1).below(background);
			}
		};
		
		int insets = 8;
		
		CRectangle bgToolRectangle = canvas.newRectangle(0, 0, toolSide, toolSide);
		bgToolRectangle.setFillPaint(Color.WHITE).addTag(rectangle).addTag(background);
		canvas.newRectangle(insets, insets, toolSide - 2*insets, toolSide - 2*insets).setPickable(false).addTag(foreground);
		
		
		CRectangle bgToolEllipse = canvas.newRectangle(toolSide, 0, toolSide, toolSide);
		bgToolEllipse.setFillPaint(Color.WHITE).addTag(ellipse).addTag(background);
		canvas.newEllipse(toolSide + insets, insets, toolSide - 2*insets, toolSide - 2*insets).setPickable(false).addTag(foreground);
		
		CRectangle bgToolSegment = canvas.newRectangle(2*toolSide, 0, toolSide, toolSide);
		bgToolSegment.setFillPaint(Color.WHITE).addTag(segment).addTag(background);
		canvas.newSegment(2*toolSide + insets, insets, 3*toolSide - insets, toolSide - insets).setPickable(false).addTag(foreground);
		
		foreground.setAntialiased(true);
	}
	
	protected void moveTo(double x, double y, double rx, double ry) {
		CRectangle bb = background.getBoundingBox();
		double currentRx = bb.getMinX() + rx * bb.getWidth();
		double currentRy = bb.getMinY() + ry * bb.getWidth();
		background.translateBy(x-currentRx, y-currentRy);
		foreground.translateBy(x-currentRx, y-currentRy);
	}
	
	public void setTransparencyLevel(float f) {
		background.setTransparencyFill(f);
		background.setTransparencyOutline(f);
		foreground.setTransparencyFill(f);
		foreground.setTransparencyOutline(f);
	}
}
