package fr.lri.insitu.FlowStates.device;

import java.awt.Container;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.EventObject;
import java.util.Hashtable;

import javax.swing.JFrame;

import fr.emn.reactiveinput.AbstractDevice;
import fr.emn.reactiveinput.Device;
import fr.emn.reactiveinput.DeviceUtilities;
import fr.emn.reactiveinput.OpenContext;
import fr.emn.reactiveinput.Processor;
import fr.emn.reactiveinput.SlotType;
import fr.lri.insitu.FlowStates.events.InSlotEvent;
import fr.lri.insitu.FlowStates.events.OutSlotEvent;
import fr.lri.insitu.FlowStates.sm.IConStateMachine;
import fr.lri.swingstates.debug.StateMachineVisualization;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.StateMachineListener;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.Event;

public class DSMDevice extends AbstractDevice implements StateMachineListener {

	protected IConStateMachine sm = null;
	protected Hashtable<Class<? extends InSlotEvent>, InSlotsGroup> inSlots;
	protected Hashtable<Class<? extends OutSlotEvent>, OutSlotsGroup> outSlots;
	
	private boolean showSMViz = false;
	private JFrame smviz = null;
	
	public DSMDevice(IConStateMachine sm) {
		super(sm.getName());
		this.sm = sm;
		createSlots();
	}
	
	@Override
	public Hashtable<Object, String[]> getAdvancedProperties() {
		Hashtable<Object, String[]> allProps = new Hashtable<Object, String[]>();
		allProps.put(this, new String[] {"ShowSMViz"});
		String[] smProps = null;
		if (sm != null) {
			smProps = sm.getProperties();		
		}
		if (smProps != null && smProps.length > 0) {		
			allProps.put(sm, smProps);
		}

		return allProps;
	}
	
	public void setShowSMViz(boolean b) {
		showSMViz = b;
	}
	
	public boolean getShowSMViz() {
		return showSMViz;
	}
	
	public boolean hasExternalOutput() {
		return true;
	}
	
	public boolean isCopiable() {
		return sm.isCopiable();
	}
	
	public Device copy() {
		DSMDevice copy = this;
		if (isCopiable() && sm.isCopiable()) {
			IConStateMachine smCopy = null;
			smCopy = sm.copy();
			if (smCopy != null) {
				copy = new DSMDevice(smCopy);
				smCopy.addStateMachineListener(copy);
				DeviceUtilities.setPrototype(copy, DeviceUtilities.getPrototype(this));
				DeviceUtilities.copyPropertyValues(this, copy);
			}/* else {
				copy = new DSMDevice(sm);
				sm.addStateMachineListener(copy);
				DeviceUtilities.setPrototype(copy, DeviceUtilities.getPrototype(this));
				DeviceUtilities.copyPropertyValues(this, copy);
			}*/
		}
		return copy;
	}
	
	public Processor open(OpenContext context) {
		if (showSMViz) {
			if (smviz == null)
				createViz();
			smviz.setVisible(true);
		} else {
			if (smviz != null)
				smviz.setVisible(false);
		}
		return super.open(context);
	}
	
	public void init() {
		update();
	}
	
	public void update() {
		try {
			sm.processIconEvents(this);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (SecurityException e) {
			e.printStackTrace();
		} catch (InstantiationException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		} catch (NoSuchMethodException e) {
			e.printStackTrace();
		}
	}
	
	private void createViz() {
		smviz = new JFrame(sm.getName() + " StateMachine Visualization");
		Container pane = smviz.getContentPane();
		StateMachineVisualization viz = new StateMachineVisualization(sm);
		pane.add(viz);
		smviz.pack();
	}
	
	private void createSlots() {
		if (sm != null) {
			inSlots = new Hashtable<Class<? extends InSlotEvent>, InSlotsGroup>();
			outSlots = new Hashtable<Class<? extends OutSlotEvent>, OutSlotsGroup>();
			for (State st : sm.getAllStates()) {
				for (Transition tr : st.getTransitions()) {
					createInputSlots(tr);
				}
			}
			createOutputSlots();
		}
	}
	
	private void createInputSlots(Transition tr) {
		if (tr instanceof Event &&  ((Event)tr).getClassEvent() != null && InSlotEvent.class.isAssignableFrom(((Event)tr).getClassEvent())) {
			Class<? extends InSlotEvent> evClass = (Class<? extends InSlotEvent>)((Event)tr).getClassEvent();
			InSlotsGroup slots = inSlots.get(evClass);
			if(slots == null) {
				slots = new InSlotsGroup(evClass);
				Method meth[] = evClass.getMethods();
				boolean hasSlots = false;
				for (Method m : meth) {
					if (m.getName().startsWith("getSlot")) {
						int type = getSlotType(m);
						String name = m.getName().substring(7);
						Method setter = getSetterFor(evClass, m);
						if (setter != null && type != SlotType.NULL) {
							slots.addSlot(addIn(evClass.getSimpleName() + "." + name, type), setter);
							hasSlots = true;
						}
					}
				}
				if(hasSlots) {
//					System.out.print(evClass+": ");
//					for (int i = 0; i < slots.getSlots().size(); i++) {
//						System.out.print(slots.getSlots().get(i).getName()+", ");	
//					}
//					System.out.println();
					inSlots.put(evClass, slots);
				}
			}
		}
	}
	
	private void createOutputSlots() {
		Class<? extends OutSlotEvent> outs[] = sm.getOutputTypes();
		if (outs == null) return;
		for (Class<? extends OutSlotEvent> out : outs) {
				OutSlotsGroup slots = outSlots.get(out);
				if(slots == null) {
					slots = new OutSlotsGroup(out);
					Method meth[] = out.getMethods();
					boolean hasSlots = false;
					for (Method m : meth) {
						if (m.getName().startsWith("getSlot")) {
							int type = getSlotType(m);
							String name = m.getName().substring(7);
							//Method setter = getSetterFor(out, m);
							if (type != SlotType.NULL) {
								slots.addSlot(addOut(out.getSimpleName() + "." + name, type), m);
								hasSlots = true;
							}
						}
					}
					if(hasSlots) {
						outSlots.put(out, slots);
					}
				}		
		}
	}
	
	private Method getSetterFor(Class<?> evClass, Method m) {
		Method meth[] = evClass.getMethods();
		for (Method m2 : meth) {
			if (!m2.getName().equals(m.getName()) && m.getName().substring(1).equals(m2.getName().substring(1)))
				return m2;
		}
		return null;
	}
	
	private int getSlotType(Method m) {
		if (m.getReturnType() == Void.TYPE)
			return SlotType.NULL;
		return SlotType.getType(m.getReturnType());
	}
	
	public InSlotsGroup getInSlotsGroup(Transition t) {
		if(!(t instanceof Event) || ((Event)t).getClassEvent() == null) return null;
		return inSlots.get(((Event)t).getClassEvent());
	}

	public OutSlotsGroup getOutSlotsGroup(Transition t) {
		return outSlots.get(t);
	}

	public void eventOccured(EventObject eventObject) {
		OutSlotsGroup group = outSlots.get(eventObject.getClass());
		if (group != null) {
			try {
				group.updateSlots((OutSlotEvent)eventObject);
			} catch (IllegalArgumentException e) {
				e.printStackTrace();
			} catch (IllegalAccessException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
	}
	
}
