package fr.lri.insitu.FlowStates.device;

import java.lang.reflect.Constructor;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.LinkedList;

import fr.emn.reactiveinput.In;
import fr.emn.reactiveinput.SlotType;
import fr.lri.insitu.FlowStates.events.InSlotEvent;
import fr.lri.swingstates.events.VirtualEvent;

public class InSlotsGroup {

	private Class<? extends InSlotEvent> evClass;
	
	public Class<? extends InSlotEvent> getEvClass() {
		return evClass;
	}

	private LinkedList<In> slots = new LinkedList<In>();
	private LinkedList<Method> methods = new LinkedList<Method>();
	
	public InSlotsGroup(Class<? extends InSlotEvent> eventClass) {
		evClass = eventClass;
	}
	
	public void addSlot(In slot, Method m) {
		slots.add(slot);
		methods.add(m);
	}
	
	public boolean hasSignal() {
		for (In slot : slots) {
			if (slot.hasSignal())
				return true;
		}
		return false;
	}
	
	public VirtualEvent getEvent() throws InstantiationException, IllegalAccessException, IllegalArgumentException, InvocationTargetException, SecurityException, NoSuchMethodException {
		if(!hasSignal()) return null;
		VirtualEvent event;
		Class<?> enclosingClass = evClass.getEnclosingClass();
		if (enclosingClass != null) {
			Object outerObject = enclosingClass.newInstance();			
			Constructor<? extends InSlotEvent> constructor = evClass.getDeclaredConstructor(new Class[] {enclosingClass});
			constructor.setAccessible(true);
			event = (VirtualEvent)constructor.newInstance(new Object[]{outerObject});
		} else {
			event = (VirtualEvent)evClass.newInstance();
		}
		
		for (int i = 0; i < slots.size(); i++) {
			if (slots.get(i).hasValue()) {				
				Method m = methods.get(i);
				m.invoke(event, getParameters(slots.get(i)));
				if (slots.get(i).hasSignal() && event instanceof InSlotEvent) {
					String shortName = slots.get(i).getName().substring(slots.get(i).getName().indexOf(".")+1).toLowerCase();
					((InSlotEvent)event).slotHasSignal(shortName);
				}
			}
		}
		if (event instanceof InSlotEvent) {
//			if (((InSlotEvent)event).occurs()) {
				return event;
//			}
		}
		return null;
	}
	
	private Object[] getParameters(In slot) {
		Object param[] = new Object[1];
		if (slot.getType() == SlotType.BOOLEAN)
			param[0] = slot.getBooleanValue();
		else if (slot.getType() == SlotType.DOUBLE)
			param[0] = slot.getDoubleValue();
		else if (slot.getType() == SlotType.INT)
			param[0] = slot.getIntValue();
		else if (slot.getType() == SlotType.STRING)
			param[0] = slot.getStringValue();
		else
			param[0] = slot.getObjectValue();
		return param;
	}

	public LinkedList<In> getSlots() {
		return slots;
	}
}
